import React from 'react';
import PropTypes from 'prop-types';
import styled from 'styled-components';

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var taggedTemplateLiteral = function (strings, raw) {
  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
};

var _templateObject = taggedTemplateLiteral(['\n    grid-area: ', '\n  '], ['\n    grid-area: ', '\n  ']),
    _templateObject2 = taggedTemplateLiteral(['\n    display: grid;\n    padding: ', ';\n    ', '\n    ', '\n    ', '\n    grid-gap: ', ';\n  '], ['\n    display: grid;\n    padding: ', ';\n    ', '\n    ', '\n    ', '\n    grid-gap: ', ';\n  ']);

var defaultBreakpoints = {
  tn: '0px', xs: '500px', sm: '768px', md: '992px',
  lg: '1200px', xl: '1440px', hg: '1920px'
};

var mediaTemplate = function mediaTemplate(direction, template) {
  if (typeof template === 'string') {
    return 'grid-template-' + direction + ': ' + template + ';';
  } else if (template === Object(template)) {
    return Object.keys(template).reduce(function (templateString, key) {
      return '\n      ' + templateString + '\n      @media (min-width: ' + (defaultBreakpoints[key] ? defaultBreakpoints[key] : key) + ') {\n        grid-template-' + direction + ': ' + template[key] + ';\n      }\n    ';
    }, '');
  }
  return 'grid-template-' + direction + ': auto;';
};

var GridChild = function GridChild(_ref) {
  var children = _ref.children,
      _area = _ref._area;

  var StyledGridChild = styled.div(_templateObject, _area);

  return React.createElement(
    StyledGridChild,
    null,
    children
  );
};

GridChild.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  _area: PropTypes.string.isRequired
};

var Grid = function Grid(_ref2) {
  var className = _ref2.className,
      children = _ref2.children,
      columns = _ref2.columns,
      rows = _ref2.rows,
      areas = _ref2.areas,
      gutter = _ref2.gutter,
      container = _ref2.container;

  var StyledGrid = styled.div(_templateObject2, container ? gutter : '0px', mediaTemplate('columns', columns), mediaTemplate('rows', rows), mediaTemplate('areas', areas), gutter);

  var gridChildren = React.Children.map(children, function (child) {
    if (child) {
      var gridChildProps = {
        '_area': child.props._area
      };

      var childPropsWithoutGridProps = Object.assign({}, child.props);

      Object.keys(gridChildProps).forEach(function (key) {
        delete childPropsWithoutGridProps[key];
      });

      var wrapped = React.createElement('div', gridChildProps, Object.assign({}, child, {
        props: childPropsWithoutGridProps
      }));

      return React.createElement(GridChild, wrapped.props);
    }
    return null;
  });
  return className ? React.createElement(
    'div',
    { className: className },
    React.createElement(
      StyledGrid,
      null,
      gridChildren
    )
  ) : React.createElement(
    StyledGrid,
    null,
    gridChildren
  );
};

Grid.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  className: PropTypes.string,
  columns: PropTypes.string,
  rows: PropTypes.string,
  areas: PropTypes.string.isRequired,
  gutter: PropTypes.string,
  container: PropTypes.bool
};

Grid.defaultProps = {
  className: null,
  columns: 'auto',
  rows: 'auto',
  gutter: '0px',
  container: false
};

var _templateObject$1 = taggedTemplateLiteral(['\n    padding: calc(', ' / 2);\n    ', '\n    ', ' \n    ', '\n    ', '\n  '], ['\n    padding: calc(', ' / 2);\n    ', '\n    ', ' \n    ', '\n    ', '\n  ']),
    _templateObject2$1 = taggedTemplateLiteral(['\n    display: flex;\n    align-content: flex-start;\n    flex-driection: ', ';\n    flex-wrap: ', ';\n    ', '\n    ', '\n    ', '\n\n    > * {\n      max-width: 100%;\n    }\n  '], ['\n    display: flex;\n    align-content: flex-start;\n    flex-driection: ', ';\n    flex-wrap: ', ';\n    ', '\n    ', '\n    ', '\n\n    > * {\n      max-width: 100%;\n    }\n  ']);

var defaultBreakpoints$1 = {
  tn: '0px', xs: '500px', sm: '768px', md: '992px',
  lg: '1200px', xl: '1440px', hg: '1920px'
};

var mediaBasis = function mediaBasis(basis) {
  if (typeof basis === 'string') {
    return 'flex-basis: ' + basis + ';';
  } else if (basis === Object(basis)) {
    return Object.keys(basis).reduce(function (basisString, key) {
      return '\n      ' + basisString + '\n      @media (min-width: ' + (defaultBreakpoints$1[key] ? defaultBreakpoints$1[key] : key) + ') {\n        flex-basis: ' + basis[key] + ';\n      }\n    ';
    }, '');
  }
  return 'flex-basis: auto;';
};

var FlexChild = function FlexChild(_ref) {
  var className = _ref.className,
      children = _ref.children,
      _grow = _ref._grow,
      _basis = _ref._basis,
      _reset = _ref._reset,
      _scroll = _ref._scroll,
      gutter = _ref.gutter,
      inline = _ref.inline;

  var StyledFlexChild = styled.div(_templateObject$1, gutter, _grow ? 'flex-grow: 1;' : '', _basis ? mediaBasis(_basis) : '', _scroll ? '\n      > * {\n        max-width: calc(100% - ' + gutter + ');\n        max-height: calc(100% - ' + gutter + ');\n        overflow: auto;\n      }\n    ' : '', !inline && _grow && _reset ? '\n      position: relative;\n      > * {\n        position: absolute;\n        width: calc(100% - ' + gutter + ');\n        height: calc(100% - ' + gutter + ');\n      }\n    ' : '');

  var renderChildWithClassName = function renderChildWithClassName() {
    return className ? React.createElement(
      'div',
      { className: className },
      children
    ) : children;
  };

  return _reset || _scroll ? React.createElement(
    StyledFlexChild,
    null,
    React.createElement(
      'div',
      null,
      renderChildWithClassName()
    )
  ) : React.createElement(
    StyledFlexChild,
    null,
    renderChildWithClassName()
  );
};

FlexChild.propTypes = {
  className: PropTypes.string,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  _grow: PropTypes.bool,
  _basis: PropTypes.oneOfType([PropTypes.string, PropTypes.objectOf(PropTypes.string)]),
  _reset: PropTypes.bool,
  _scroll: PropTypes.bool,
  inline: PropTypes.bool,
  gutter: PropTypes.string
};

FlexChild.defaultProps = {
  className: null,
  _grow: false,
  _reset: false,
  _scroll: false,
  _basis: null,
  inline: false,
  gutter: '0px'
};

var Flex = function Flex(_ref2) {
  var className = _ref2.className,
      children = _ref2.children,
      inline = _ref2.inline,
      wrap = _ref2.wrap,
      itemsCenter = _ref2.itemsCenter,
      fullHeight = _ref2.fullHeight,
      container = _ref2.container,
      gutter = _ref2.gutter;

  var StyledFlex = styled.div(_templateObject2$1, inline ? 'row' : 'column', wrap && inline ? 'wrap' : 'no-wrap', itemsCenter ? 'align-items: center;' : '', fullHeight ? 'min-height: 100%;' : '', container ? '\n      margin: 0;\n      padding: calc(' + gutter + ' / 2);\n    ' : '\n      margin: calc(-' + gutter + ' / 2);\n    ');

  var flexChildren = React.Children.map(children, function (child) {
    // Wrap any children in a div to prevent potential css flex layout overrides.
    if (child) {
      var flexChildProps = Object.keys(FlexChild.defaultProps).reduce(function (props, key) {
        if (key !== 'className') {
          props[key] = child.props[key];
        }
        return props;
      }, {});

      var childPropsWithoutFlexProps = Object.assign({}, child.props);

      Object.keys(flexChildProps).forEach(function (key) {
        delete childPropsWithoutFlexProps[key];
      });

      var wrapped = React.createElement('div', flexChildProps, Object.assign({}, child, {
        props: childPropsWithoutFlexProps
      }));

      return React.createElement(FlexChild, _extends({}, wrapped.props, { gutter: gutter, isInline: inline }));
    }
    return null;
  });

  return className ? React.createElement(
    'div',
    { className: className },
    React.createElement(
      StyledFlex,
      null,
      flexChildren
    )
  ) : React.createElement(
    StyledFlex,
    null,
    flexChildren
  );
};

Flex.propTypes = {
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  className: PropTypes.string,
  inline: PropTypes.bool,
  wrap: PropTypes.bool,
  itemsCenter: PropTypes.bool,
  fullHeight: PropTypes.bool,
  container: PropTypes.bool,
  gutter: PropTypes.string
};

Flex.defaultProps = {
  className: null,
  inline: false,
  wrap: false,
  itemsCenter: false,
  fullHeight: false,
  container: false,
  gutter: '0px'
};

export { Grid, Flex };
